"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.massageIndexOptions = exports.command = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const logger_1 = require("../../logger");
const path_1 = __importDefault(require("path"));
const fs_1 = require("fs");
const command_1 = require("../guard/command");
const kubevious_1 = require("../../types/kubevious");
const yaml_1 = __importDefault(require("yaml"));
const path_2 = require("../../utils/path");
const screen_1 = require("../../screen");
function command(dir, options) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        logger_1.logger.info("[Dir] ", dir);
        const guardResult = yield (0, command_1.command)([dir], {
            ignoreUnknown: true,
            ignoreNonK8s: true,
            stream: false,
            skipApplyCrds: false,
            liveK8s: false,
            includeRemoteTargets: false,
            skipLocalRules: false,
            skipRemoteRules: true,
            areNamespacesSpecified: false,
            namespaces: [],
            skipClusterScope: false,
            skipCommunityRules: true,
            skipRuleLibraries: true,
        });
        const manifestPackage = guardResult.manifestPackage;
        const libraryFilePath = path_1.default.join(dir, 'index.yaml');
        const libraryObject = {
            apiVersion: `${kubevious_1.KUBEVIOUS_API_NAME}/${kubevious_1.KUBEVIOUS_API_VERSION}`,
            kind: kubevious_1.KubeviousKinds.Library,
            metadata: {
                name: 'community'
            },
            spec: {
                rules: []
            }
        };
        const rules = guardResult.localK8sRegistry.query({
            apiName: kubevious_1.KUBEVIOUS_API_NAME,
            kind: kubevious_1.KubeviousKinds.ClusterRule
        });
        for (const rule of rules) {
            const ruleConfig = rule.config.spec;
            logger_1.logger.info("LIBRARY RULE: %s", rule.idKey);
            const category = (0, path_2.makeRelativePath)(path_1.default.dirname(rule.source.source.path), dir);
            const rulePath = (0, path_2.makeRelativePath)(rule.source.source.path, dir);
            libraryObject.spec.rules.push({
                name: rule.id.name,
                path: rulePath,
                category: category,
                summary: (_a = ruleConfig.summary) !== null && _a !== void 0 ? _a : "",
            });
        }
        libraryObject.spec.rules = the_lodash_1.default.orderBy(libraryObject.spec.rules, [x => x.name, x => x.path]);
        const libraryResult = {
            categories: []
        };
        {
            const categoryDict = {};
            for (const rule of libraryObject.spec.rules) {
                if (!categoryDict[rule.category]) {
                    categoryDict[rule.category] = {
                        name: rule.category,
                        rules: []
                    };
                }
                categoryDict[rule.category].rules.push(rule);
            }
            for (const category of the_lodash_1.default.values(categoryDict)) {
                category.rules = the_lodash_1.default.orderBy(category.rules, x => x.name);
            }
            libraryResult.categories =
                the_lodash_1.default.chain(categoryDict)
                    .values()
                    .orderBy(x => x.name)
                    .value();
        }
        if (guardResult.success) {
            yield fs_1.promises.writeFile(libraryFilePath, yaml_1.default.stringify(libraryObject, {
                toStringDefaults: {
                    indent: 2,
                    indentSeq: false
                }
            }));
            yield setupDocs(dir, libraryResult);
        }
        const success = guardResult.success;
        return {
            success,
            manifestPackage,
            guardCommandData: guardResult,
            libraryDir: dir,
            libraryPath: libraryFilePath,
            library: libraryResult,
            rules,
        };
    });
}
exports.command = command;
function massageIndexOptions(options) {
    return {};
}
exports.massageIndexOptions = massageIndexOptions;
const README_BEGIN = '[//]: # (BEGIN_RULES_DESCRIPTION)';
const README_END = '[//]: # (END_RULES_DESCRIPTION)';
function setupDocs(dir, libraryResult) {
    return __awaiter(this, void 0, void 0, function* () {
        const readmePath = path_1.default.join(dir, 'README.md');
        if (!(0, fs_1.existsSync)(readmePath)) {
            return;
        }
        let contents = (yield fs_1.promises.readFile(readmePath)).toString();
        const beginIndex = contents.indexOf(README_BEGIN);
        logger_1.logger.info("[beginIndex] %s", beginIndex);
        if (beginIndex == -1) {
            return;
        }
        const endIndex = contents.indexOf(README_END);
        logger_1.logger.info("[endIndex] %s", endIndex);
        if (endIndex == -1) {
            return;
        }
        const deleteStartIndex = beginIndex + README_BEGIN.length;
        if (endIndex <= deleteStartIndex) {
            return;
        }
        contents =
            contents.substring(0, deleteStartIndex) +
                "\n" +
                generateMDDocs(libraryResult) +
                "\n" +
                contents.substring(endIndex);
        yield fs_1.promises.writeFile(readmePath, contents);
    });
}
function generateMDDocs(libraryResult) {
    let contents = '';
    contents += '\n';
    contents += '[//]: # (!!! DO NOT EDIT. AUTO-GENERATED WITH:)\n';
    contents += '[//]: # ($ kubevious index-library .)\n';
    contents += '[//]: # (OR PRE-COMMIT HOOK)\n';
    contents += '[//]: # ($ kubevious install-git-hook rule-library .)\n';
    contents += '\n';
    for (const category of libraryResult.categories) {
        contents += `### ${screen_1.OBJECT_ICONS.ruleCategory.get()} ${the_lodash_1.default.toUpper(category.name)}`;
        contents += '\n';
        for (const rule of category.rules) {
            contents += `${screen_1.OBJECT_ICONS.rule.get()} **[${rule.name}](${rule.path})**.`;
            contents += '\n';
            if (rule.summary.length > 0) {
                contents += rule.summary;
                contents += '\n';
            }
            contents += '\n';
        }
    }
    return contents;
}
//# sourceMappingURL=command.js.map