"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidationProcessor = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const the_promise_1 = require("the-promise");
const compiler_1 = require("@kubevious/kubik/dist/processors/compiler");
const sync_scope_builder_1 = require("../../query-spec/sync-scope-builder");
const scope_builder_1 = require("../../query-spec/scope-builder");
const rule_helpers_1 = require("../../helpers/rule-helpers");
class ValidationProcessor {
    constructor(src, executionContext) {
        this._src = src;
        this._executionContext = executionContext;
        this._runnable = null;
        this._logger = executionContext.logger.sublogger('ValidationProcessor');
    }
    prepare() {
        const result = {
            success: false,
            messages: [],
        };
        return this._loadModule()
            .then((runnable) => {
            this._runnable = runnable;
        })
            .then(() => {
            this._validate();
            result.success = result.messages.length == 0;
        })
            .catch((reason) => {
            result.success = false;
            this._addError(result.messages, reason.message);
        })
            .then(() => result);
    }
    _loadModule() {
        return the_promise_1.Promise.resolve().then(() => {
            const compilerValues = {
                item: null,
                config: null,
                cache: null,
                values: null,
                error: null,
                warning: null,
                mark: null,
                helpers: null,
            };
            for (const x of the_lodash_1.default.keys(scope_builder_1.TARGET_QUERY_BUILDER_DICT)) {
                compilerValues[x] = null;
            }
            const compiler = new compiler_1.Compiler(this._src, 'RULE_VALIDATOR', compilerValues);
            return compiler.compile();
        });
    }
    _validate() { }
    execute(item, cache, values) {
        this._logger.info("[execute] item: %s", item.manifest.idKey);
        this._logger.info("[execute] item: %s", item.manifest.idKey);
        const result = {
            success: false,
            messages: [],
            validation: {
                hasErrors: false,
                hasWarnings: false,
                errorMsgs: {},
                warnMsgs: {},
            },
        };
        return the_promise_1.Promise.resolve()
            .then(() => {
            const valueMap = {
                item: item,
                values: values,
                config: item.config,
                cache: cache,
                helpers: rule_helpers_1.RULE_HELPERS,
                error: (msg) => {
                    result.validation.hasErrors = true;
                    if (msg) {
                        result.validation.errorMsgs[msg] = true;
                    }
                },
                warning: (msg) => {
                    result.validation.hasWarnings = true;
                    if (msg) {
                        result.validation.warnMsgs[msg] = true;
                    }
                },
                mark: (kind) => {
                    if (!result.validation.marks) {
                        result.validation.marks = {};
                    }
                    result.validation.marks[kind] = true;
                },
            };
            this._setupQueryBuilders(valueMap, item);
            // console.log("HEADERS: ", _.keys(valueMap))
            return this._runnable.run(valueMap);
        })
            .then(() => {
            result.success = true;
        })
            .catch((reason) => {
            this._logger.info("[execute] error: ", reason);
            result.success = false;
            this._addError(result.messages, reason.message);
        })
            .then(() => result);
    }
    _setupQueryBuilders(valueMap, item) {
        const queryScope = (0, sync_scope_builder_1.buildQueryableScope)(this._executionContext, { namespace: item.namespace });
        for (const key of the_lodash_1.default.keys(queryScope)) {
            valueMap[key] = queryScope[key];
        }
    }
    _addError(list, msg) {
        list.push(msg);
    }
}
exports.ValidationProcessor = ValidationProcessor;
//# sourceMappingURL=processor.js.map