"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildQueryableScope = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const scope_builder_1 = require("./scope-builder");
function buildQueryableScope(executionContext, limiter) {
    const syncQueryBuilder = {};
    for (const key of the_lodash_1.default.keys(scope_builder_1.TARGET_QUERY_BUILDER_DICT)) {
        const builder = scope_builder_1.TARGET_QUERY_BUILDER_DICT[key];
        const syncQuery = new SyncQueryBuilder(executionContext, builder);
        syncQueryBuilder[key] = syncQuery.wrap(limiter);
    }
    return syncQueryBuilder;
}
exports.buildQueryableScope = buildQueryableScope;
class SyncQueryBuilder {
    constructor(executionContext, queryBuilder) {
        this._executionContext = executionContext;
        this._queryBuilder = queryBuilder;
        this._logger = this._executionContext.logger.sublogger("SyncQueryBuilder");
    }
    wrap(limiter) {
        const wrapper = (...args) => {
            const queryTarget = this._queryBuilder.apply(null, args);
            queryTarget.many = () => {
                return this._executeQuery(queryTarget, limiter);
            };
            queryTarget.single = () => {
                var _a;
                return (_a = the_lodash_1.default.head(this._executeQuery(queryTarget, limiter))) !== null && _a !== void 0 ? _a : null;
            };
            queryTarget.count = () => {
                return this._executeQuery(queryTarget, limiter).length;
            };
            return queryTarget;
        };
        return wrapper;
    }
    _executeQuery(queryTarget, limiter) {
        var _a;
        const result = this._executionContext.queryExecutor.execute(queryTarget, limiter);
        return (_a = result.items) !== null && _a !== void 0 ? _a : [];
    }
}
//# sourceMappingURL=sync-scope-builder.js.map